/**********************************************************************
**   Proc: b64mime6.p
**   Desc: Really, Really Fast Base64 Conversion (plus MIME headers)
**   Date: 6 July 2001
** Author: Peter J. Graybash III (pjg3@luther-rose.com)
** Changes:
**   20010707: add code to fix slashes in filenames (&FIX_SLASHES)
**   20010709: combine MIME header puts into one statement
**             change email address in header
**   20010710: borrowed the read 1 / write 6 and tweaked it!
**********************************************************************/

/** Remove the -not if you can't trust the incoming file names... **/
&scoped FIX_SLASHES-not

&if OPSYS EQ 'UNIX' &then
  &scoped OS_SLASH    '/'
  &scoped WRONG_SLASH '\\\\'
&else
  &scoped OS_SLASH    '\'
  &scoped WRONG_SLASH '/'
&endif

/** Parameter Definitions **/
def input param cInputFile  as char no-undo.
def input param cOutputFile as char no-undo.

/** Working Variables **/
def var rrr as raw  no-undo.
def var bbb as char no-undo.
def var fff as char no-undo.
def var ccc as char no-undo.
def var iii as int  no-undo.

/** Build MIME Boundary Using Current Date & Time & Ticks **/
assign
  &if defined(FIX_SLASHES) &then
    cInputFile  = replace(cInputFile, {&WRONG_SLASH}, {&OS_SLASH})
    cOutputFile = replace(cOutputFile, {&WRONG_SLASH}, {&OS_SLASH})
  &endif
  fff = entry(num-entries(cInputFile, {&OS_SLASH}), cInputFile, {&OS_SLASH})
  bbb = substitute('b64mime_&1_&2.&3', int(today), time, etime mod 1000).

/** Open Input & Output File Streams **/
input  from value(cInputFile) binary no-echo no-map no-convert.
output to   value(cOutputFile).

/*
/** Output MIME Headers : Thanks to Scott Auge ! **/
put control (
    'MIME-Version: 1.0~n' +
    'Content-Transfer-Encoding: 7bit~n' +
    'Content-type: multipart/mixed; boundary=' + bbb +
    '~n~n--' + bbb +
    '~nContent-type: text/html name=' + fff +
    '~nContent-Transfer-Encoding: base64~n~n').
*/

/** Magic File Encoder **/
length(rrr) = 342.
repeat:
  import unformatted rrr.
  assign
      ccc = string(rrr)
      iii = length(ccc).
  put control
      (if iii gt 7   then substring(ccc,   7, 76, 'RAW') + '~r~n' else '') +
      (if iii gt 83  then substring(ccc,  83, 76, 'RAW') + '~r~n' else '') +
      (if iii gt 159 then substring(ccc, 159, 76, 'RAW') + '~r~n' else '') +
      (if iii gt 235 then substring(ccc, 235, 76, 'RAW') + '~r~n' else '') +
      (if iii gt 311 then substring(ccc, 311, 76, 'RAW') + '~r~n' else '') +
      (if iii gt 387 then substring(ccc, 387, 76, 'RAW') + '~r~n' else '').
end.
length(rrr) = 0.

/*
/** Output MIME Trailers **/
put control '~n--' + bbb + '~n'.
*/

/** Close Input & Output File Streams **/
input  close.
output close.
/*********************************************************************/
