
/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


/* BulkLoad.p
 * 
 * Written by Scott Auge
 *
 * Loads in a list of email addresses from a file of email addresses
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/ezine/src/RCS/BulkLoad.p,v 1.2 2001/11/09 00:19:59 sauge Exp sauge $" NO-UNDO.

DEF VAR lFileName          AS CHARACTER NO-UNDO.
DEF VAR lEmailAddress      AS CHARACTER NO-UNDO.
DEF VAR lPersonID          AS CHARACTER NO-UNDO.
DEF VAR lContactID         AS CHARACTER NO-UNDO.
DEF VAR lMailListID        AS CHARACTER NO-UNDO.
DEF VAR lListMemberID      AS CHARACTER NO-UNDO.
DEF VAR lSecUserID         AS CHARACTER NO-UNDO.
DEF VAR lConfFileName      AS CHARACTER NO-UNDO.
DEF VAR lValue             AS CHARACTER NO-UNDO.
DEF VAR lError             AS CHARACTER NO-UNDO.

DEF VAR lFirstName         AS CHARACTER NO-UNDO.
DEF VAR lLastName          AS CHARACTER NO-UNDO.
DEF VAR lDefaultPassword   AS CHARACTER NO-UNDO.

{Error.i}

/* Determine name of the mailing list and if it exists.  If not, prep the */
/* data for integrity.                                                    */

ASSIGN lConfFileName = OS-GETENV("CONFFILE").

RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "List",
                  INPUT "ListName",
                  OUTPUT lValue).


/* Determine if the Mail List record exists, if not, go ahead and create it */

FIND MailList NO-LOCK
WHERE MailList.Name = lValue
NO-ERROR.

IF NOT AVAILABLE MailList THEN DO:

  RUN records/maillist/CreateMailList.p (INPUT lValue,
                                         OUTPUT lMailListID,
                                         OUTPUT lError).

  IF lError <> {&NOERROR} THEN DO:

    PUT UNFORMATTED ("MailList:" + lError) SKIP.
    RETURN.

  END. /* IF lError <> {&NOERROR} */

END.  /* IF NOT AVAILABLE MailList */

ELSE ASSIGN lMailListID = MailList.MailListID.

/* Determine the default password to assign the accounts */

RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "Password",
                  INPUT "Default",
                  OUTPUT lDefaultPassword).

/* Open up the file that has the email addresses and load em in */

ASSIGN lFileName = OS-GETENV("MAILLIST").

INPUT FROM VALUE(lFileName).

REPEAT:

  IMPORT UNFORMATTED lEmailAddress.

  /* Create a Contact record.  This is needed to link into the Person record */
  /* as it is a relational child to the Person record.                       */
  
  RUN records/contact/CreateContact.p (INPUT "",
                                       INPUT "",
                                       INPUT lEmailAddress,
                                       INPUT "",
                                       INPUT "",
                                       OUTPUT lContactID,
                                       OUTPUT lError).

  IF lError <> {&NOERROR} THEN DO:
  
    PUT UNFORMATTED ("Contact:" + lError) SKIP.
    RETURN.

  END. /* IF lError <> {&NOERROR} */

  /* Create the Person record.  This will be linked into the EZine Mailing */
  /* list record.                                                          */

  RUN records/person/CreatePerson.p (INPUT lFirstName,
                                     INPUT lLastName,
                                     INPUT "",
                                     INPUT lContactID,
                                     OUTPUT lPersonID,
                                     OUTPUT lError).
  
  IF lError <> {&NOERROR} THEN DO:

    PUT UNFORMATTED  ("Person:" + lError) SKIP.
    RETURN.
  
  END. /* IF lError <> {&NOERROR} */

  /* Make this a secure user */

  RUN records/secuser/CreateSecUser.p (INPUT lPersonID,
                                       INPUT lEmailAddress,
                                       INPUT lDefaultPassword,
                                       INPUT "",
                                       OUTPUT lSecUserID,
                                       OUTPUT lError).


  /* Add the person to the Mailing List */

  RUN records/maillist/CreateListMember.p (INPUT lMailListID,
                                           INPUT lPersonID,
                                           OUTPUT lListMemberID,
                                           OUTPUT lError).

  IF lError <> {&NOERROR} THEN DO:

    PUT UNFORMATTED  ("MailListMember:" + lError) SKIP.
    RETURN.
  
  END. /* IF lError <> {&NOERROR} */

END. /* REPEAT */

INPUT CLOSE.

