
/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


/* PullMail.p
 *
 * Pull in mail from the given email address
 */


DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/ezine/src/RCS/PullMail.p,v 1.1 2001/11/09 00:19:59 sauge Exp sauge $" NO-UNDO.

DEF INPUT PARAMETER pAccountID        AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pAccountPassword  AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pAccountMachine   AS CHARACTER NO-UNDO.
DEF INPUT PARAMETER pDeleteServerMail AS LOGICAL NO-UNDO.

DEF VAR lMachineHdl  AS HANDLE NO-UNDO.
DEF VAR lResult      AS CHARACTER NO-UNDO.
DEF VAR lMessageIDs  AS CHARACTER NO-UNDO.

DEF VAR lNumberOfEmails    AS INTEGER NO-UNDO.
DEF VAR lCurrentMessageID  AS INTEGER NO-UNDO.

{pop3api.i}

/* Connect to the machine.  Determine if we are OK with the result */

RUN ConnectToServer(INPUT pAccountMachine,
                    OUTPUT lMachineHdl,
                    OUTPUT lResult).

/* Login as the user */

RUN Login (INPUT lMachineHdl,
           INPUT pAccountID
           INPUT pAccountPassword,
           OUTPUT lResult).

/* Determine the number of messages waiting and their IDs */

RUN Listmessages (INPUT lMachineHdl,
                  OUTPUT lMessageIDs).

ASSIGN lNumberOfEmails = NUM-ENTRIES(lMessageIDs).

/* Pull in the email, then delete the email off the machine */

DO lCurrentMessageID = 1 TO lNumberOfEmails:

  /* Call an internal procedure that will take the message out of */
  /* raw memory and place it into a record.                       */ 

  RUN GetMessage (INPUT lMachineHdl,
                  INPUT lCurrentMessageID).

END. /* DO lCurrentMessageID = 1 TO lNumberOfEmails: */

/* Delete the emails IDed */

IF pDeleteServerMail THEN
  DO lCurrentMessageID = 1 TO lNumberOfEmails:

    RUN DeleteMessage(INPUT STRING (lCurremtMessageID),
                      OUTPUT lResult).

  END. /* DO lCurrentMessageID = 1 TO lNumberOfEmails: */

/* Close the connection */

RUN LogOut (INPUT lMachineHdl,
            OUTPUT lResult).


/************************* Internal Procedures *************************/

/* Uses the POP3 api to retrieve the text of an email message.  It then */
/* parses the text to create and fill in an Email record.               */

PROCEDURE GetMessage:

  DEF INPUT PARAMETER pMachineHdl        AS HANDLE NO-UNDO.
  DEF INPUT PARAMETER pCurrentMessageID  AS INTEGER NO-UNDO.

  DEF VAR lMessageText       AS CHARACTER NO-UNDO.

  /* Retrieve the message off the server */

  RUN RetrieveMessage (INPUT pMachineHdl,
                       INPUT STRING (pCurrentMessageID),
                       OUTPUT lMessageText).

  /* Put the message into a Email record within the database */

  RUN CreateEmailRecord (INPUT lMessageText).

END. /* PROCEDURE GetMessage */

/* Creates the email record and disects the headers into the proper places. */

PROCEDURE CreateEmailRecord:

  DEF INPUT PARAMETER lMessageText   AS CHARACTER NO-UNDO.

  CREATE Email.

  ASSIGN
  Email.EmailID = MakeID (NEXT-VALUE(EmailID)).

  ASSIGN 
  Email.RawEmail = lMessageText
  Email.State = "RawRcvd".

  RUN EmailParse (BUFFER Email).

END. /* PROCEDURE CreateEmailRecord */

FUNCTION ParseHeader RETURNS CHARACTER (INPUT HeaderTag AS CHARACTER,
                                        INPUT RawMessage AS CHARACTER):

  DEF VAR i AS INTEGER NO-UNDO.
  DEF VAR t AS CHARACTER NO-UNDO.

  ASSIGN i = INDEX (RawMessage, HeaderTag).

  ASSIGN t =  

END. /* FUNCTION ParseHeader */

PROCEDURE EmailParse:

  DEF PARAMETER BUFFER bEmail FOR Email.

  ASSIGN 
  bEmail.Subject = ParseHeader(INPUT "Subject:", INPUT bEmail.RawEmail)
  bEmail.Body
  bEmail.FromAddress = ParseHeader(INPUT "From:", INPUT bEmail.RawEmail)
  bEmail.ToAddress = ParseHeader(INPUT "To:", INPUT bEmail.RawEmail)
  bEmail.CCAddress = 

END. /* PROCEDURE EmailParse */


