
/*
 * Written by Scott Auge scott_auge@yahoo.com sauge@amduus.com
 * Copyright (c) 2001 Amduus Information Works, Inc.  www.amduus.com
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Amduus Information Works
 *      and its contributors.
 * 4. Neither the name of Amduus Information Works, Inc. nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY AMDUUS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AMDUUS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


/* bat_sendmessage.p
 *
 * Written by Scott Auge
 */

DEF VAR RCSVersion AS CHARACTER INIT "$Header: /home/appl/ezine/src/RCS/bat_SendMessage.p,v 1.2 2001/11/09 00:19:59 sauge Exp sauge $" NO-UNDO.

DEF VAR lListName          AS CHARACTER NO-UNDO.
DEF VAR lConfFileName      AS CHARACTER NO-UNDO.
DEF VAR lMailHub           AS CHARACTER NO-UNDO.
DEF VAR lFromAddress       AS CHARACTER NO-UNDO.
DEF VAR oSuccessful        AS CHARACTER NO-UNDO.
DEF VAR vMessage           AS CHARACTER NO-UNDO.
DEF VAR lLogFile           AS CHARACTER NO-UNDO.
DEF VAR lPause             AS CHARACTER NO-UNDO.
DEF VAR lThrottle          AS INTEGER NO-UNDO.

/* Pick up where the configuration file is at */

ASSIGN lConfFileName = OS-GETENV("CONFFILE").

/* Determine the throttle between emails cuz a large list can over tax a */
/* server's resources.                                                   */

RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "Command",
                  INPUT "Pause",
                  OUTPUT lPause).

ASSIGN lThrottle = INTEGER (lPause).

/* Determine Log File */

RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "Logging",
                  INPUT "LogFile",
                  OUTPUT lLogFile).

OUTPUT TO VALUE(lLogFile).

/* Determine name of the mailing list and if it exists.  If not, prep the */
/* data for integrity.                                                    */


RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "List",
                  INPUT "ListName",
                  OUTPUT lListname).


FIND MailList NO-LOCK
WHERE MailList.Name = lListName
NO-ERROR.

/* Determine the Mail Hub we are sending this to */

RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "Mail",
                  INPUT "MailHub",
                  OUTPUT lMailHub).

/* Determine who the mail is from in case a system account is running */
/* this program.                                                      */

RUN conf/inird.p (INPUT lConfFileName,
                  INPUT "Mail",
                  INPUT "FromAddress",
                  OUTPUT lFromAddress).
/* Look for emails ready to go */

FOR EACH Email EXCLUSIVE-LOCK
WHERE Email.ToAddress = lListName
  AND Email.State = "Send":

  /* Send email out to each member on the mail list */

  FOR EACH MailListMember OF MailList NO-LOCK:

    FIND Person OF MailListMember NO-LOCK NO-ERROR.

    FIND Contact OF Person NO-LOCK NO-ERROR.

    RUN bat_smtpmail.p (INPUT lMailHub,
                        INPUT Contact.Email,
                        INPUT lFromAddress,
                        INPUT "",
                        INPUT "",
                        INPUT "",
                        INPUT Email.Subject,
                        INPUT Email.Body,
                        INPUT "",
                        INPUT "",
                        OUTPUT oSuccessful,
                        OUTPUT vMessage).

    PAUSE lThrottle.

  END. /* FOR EACH MailListMember OF MailList */

  ASSIGN Email.State = "Sent".

END. /* FOR EACH Email */

