/* file: pop3api.i
 * Written by Scott Auge
 * sauge@aezinc.com
 *
 * This file is released into the public domain.
 *
 * Written by Scott Auge
 * Perform operations on POP3 mail servers
 *
 * Use this file as an include or a persistance procedure.
 */

DEF VAR RCSVersionpop3apii AS CHARACTER INIT "$Header: /home/appl/ezine/src/RCS/pop3api.i,v 1.1 2001/09/28 02:18:59 sauge Exp $" NO-UNDO.

{sdk/pop3.i}
{sdk/memptr.i}

/****************************************************************/
/* Create the socket, connect to server.                        */
/****************************************************************/

PROCEDURE ConnectToServer:

  DEF INPUT PARAMETER MailServer AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER hHandle AS HANDLE NO-UNDO.
  DEF OUTPUT PARAMETER Result AS INTEGER NO-UNDO.

  DEF VAR hSocket AS HANDLE NO-UNDO.
  DEF VAR lConnectParameters AS CHARACTER NO-UNDO.

  CREATE SOCKET hSocket.
  hSocket:SET-SOCKET-OPTION ("TCP-NODELAY", "FALSE").
  hSocket:SET-SOCKET-OPTION ("SO-LINGER", "FALSE").

  ASSIGN lConnectParameters = '-H ' + MailServer + ' -S pop3'.

  hSocket:CONNECT(lConnectParameters) NO-ERROR.

  IF hSocket:CONNECTED() = FALSE THEN DO:
    ASSIGN Result = {&NOCONNECT}.
    RETURN.
  END.

  ASSIGN Result = {&NOERROR}
         hHandle = hSocket.

END. /* PROCEDURE ConnectToServer */

PROCEDURE DisconnectFromServer:

  DEF INPUT PARAMETER hSocket AS HANDLE NO-UNDO.

  hSocket:DISCONNECT().

END.  /* PROCEDURE DisconnectFromServer */

/****************************************************************/
/* Create the socket, connect to server.                        */
/****************************************************************/

PROCEDURE Login:

  DEF INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
  DEF INPUT PARAMETER pUserID  AS CHARACTER NO-UNDO.
  DEF INPUT PARAMETER pPassword AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pMessageText AS CHARACTER NO-UNDO.

  DEF VAR lPOP3Command AS CHARACTER NO-UNDO.

  ASSIGN lPOP3Command = "USER " + pUserID + "~n".
  RUN lmemput (mData, lPOP3Command).
  hSocket:WRITE(mData,1,LENGTH(lPOP3Command)) NO-ERROR.

  RUN ProcessServerResponse (INPUT hSocket, OUTPUT pMessageText).

  ASSIGN lPOP3Command = "PASS " + pPassword + "~n".
  RUN lmemput (mData, lPOP3Command).
  hSocket:WRITE(mData,1,LENGTH(lPOP3Command)) NO-ERROR.

  RUN ProcessServerResponse (INPUT hSocket, OUTPUT pMessageText).

END. /* PROCEDURE Login */

/****************************************************************/
/* List messages available on the server.                       */
/****************************************************************/

PROCEDURE ListMessages:

  DEF INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
  DEF OUTPUT PARAMETER pMessageText AS CHARACTER NO-UNDO.

  DEF VAR lPOP3Command AS CHARACTER NO-UNDO.
  DEF VAR i AS INTEGER NO-UNDO.
  DEF VAR t AS CHARACTER NO-UNDO.

  ASSIGN lPOP3Command = "LIST~n".
  RUN lmemput (mData, lPOP3Command).
  hSocket:WRITE(mData,1,LENGTH(lPOP3Command)) NO-ERROR.

  RUN ProcessServerResponse (INPUT hSocket, OUTPUT pMessageText).


  ASSIGN pMessageText = trim (pMessageText).

  /*--- remove hard carriage returns with spaces ---*/
  DO WHILE (INDEX(pMessageText, CHR(13)) <> 0):
    ASSIGN SUBSTRING(pMessageText, INDEX(pMessageText, CHR(13))) = " ".
  END.
  /*--- remove hard carriage returns with spaces ---*/
  DO WHILE (INDEX(pMessageText, CHR(10)) <> 0):
    ASSIGN SUBSTRING(pMessageText, INDEX(pMessageText, CHR(10))) = ",".
  END.

  DO i = 2 TO NUM-ENTRIES(pMessageText):
    ASSIGN t = t + ENTRY(1, ENTRY(i, pMessageText), " ") + ",".
  END.
  ASSIGN pMessageText = SUBSTRING(t, 1, LENGTH(t) - 3).

END. /* PROCEDURE ListMessages */

/****************************************************************/
/* Retreive message from the server.                            */
/****************************************************************/

PROCEDURE RetrieveMessage:

  DEF INPUT PARAMETER hSocket as HANDLE NO-UNDO.
  DEF INPUT PARAMETER pMessageID AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER pMessageText AS CHARACTER NO-UNDO.

  DEF VAR lPOP3Command AS CHARACTER NO-UNDO.

  ASSIGN lPOP3Command = "RETR " + pMessageID + "~n".
  RUN lmemput (mData, lPOP3Command).
  hSocket:WRITE(mData,1,LENGTH(lPOP3Command)) NO-ERROR.

  RUN ProcessServerResponse (INPUT hSocket, OUTPUT pMessageText).

END. /* PROCEDURE RetrieveMessage */

/****************************************************************/
/* Delete a message from the server.                            */
/****************************************************************/

PROCEDURE DeleteMessage:

  DEF INPUT PARAMETER hSocket as HANDLE NO-UNDO.
  DEF INPUT PARAMETER pMessageID AS CHARACTER NO-UNDO.
  DEF OUTPUT PARAMETER lServerResponse AS CHARACTER NO-UNDO.

  DEF VAR lPOP3Command AS CHARACTER NO-UNDO.

  ASSIGN lPOP3Command = "DELE " + pMessageID + "~n".
  RUN lmemput (mData, lPOP3Command).
  hSocket:WRITE(mData,1,LENGTH(lPOP3Command)) NO-ERROR.

  RUN ProcessServerResponse (INPUT hSocket, OUTPUT lServerResponse).

END. /* PROCEDURE DeleteMessage */

/****************************************************************/
/* Logout of the server.                                        */
/****************************************************************/

PROCEDURE Logout:

  DEF INPUT PARAMETER hSocket as HANDLE NO-UNDO.
  DEF OUTPUT PARAMETER lServerResponse AS CHARACTER NO-UNDO.

  DEF VAR lPOP3Command AS CHARACTER NO-UNDO.

  ASSIGN lPOP3Command = "QUIT~n".
  RUN lmemput (mData, lPOP3Command).
  hSocket:WRITE(mData,1,LENGTH(lPOP3Command)) NO-ERROR.

  RUN ProcessServerResponse (INPUT hSocket, OUTPUT lServerResponse).

END.

/****************************************************************/
/* Get a response from the server.                              */
/****************************************************************/

PROCEDURE ProcessServerResponse:

  DEF INPUT PARAMETER hSocket AS HANDLE NO-UNDO.
  DEF OUTPUT PARAMETER lResponse AS CHARACTER NO-UNDO.

  DEF VAR i AS INTEGER INIT 0 NO-UNDO.

  /****************************************************************/
  /* Sometimes we may need to wait a little bit for the server to */
  /* respond so we loop a bit till we get something interesting.  */
  /* If we dont, we just bail out of here.                        */
  /****************************************************************/

  DO WHILE i < 6:
    IF hSocket:GET-BYTES-AVAILABLE() <> 0 THEN LEAVE.
    PAUSE 1 NO-MESSAGE.
    ASSIGN i = i + 1.
  END.

  IF hSocket:GET-BYTES-AVAILABLE() = 0 THEN RETURN.

  /****************************************************************/
  /* We discover we have something to read from the socket, lets  */
  /* see what we got!  Pop it into raw memory, then put it into   */
  /* a 4GL varible.                                               */
  /****************************************************************/

  RUN lmemset (mData, " ").
  hSocket:READ (mData, 1, GET-SIZE(mData), READ-AVAILABLE).

  RUN lmemget (mData, OUTPUT LResponse).

END. /* PROCEDURE ProcessServerResponse */

